//////////////////////////////////////////////////////////////////////////////////
// Project Name: [ CDX Class Library - CDX.lib ]
// Source File:  [ CDXInput Implementation ]
// Author:       [ Danny Farley - danny@jags.co.uk ]
// Revision:     [ 1.4 ]
//////////////////////////////////////////////////////////////////////////////////
#include "CDX.h"

//////////////////////////////////////////////////////////////////////////////////
// CDXInput Constructor
//////////////////////////////////////////////////////////////////////////////////
CDXInput::CDXInput()
{
	m_lpDI = NULL;
	m_lpDIDKeyboard = NULL;
	m_lpDIDMouse = NULL;
}

//////////////////////////////////////////////////////////////////////////////////
// CDXInput Destructor
//////////////////////////////////////////////////////////////////////////////////
CDXInput::~CDXInput()
{
	RELEASE(m_lpDIDKeyboard);
	RELEASE(m_lpDIDMouse);
	RELEASE(m_lpDI);
}

//////////////////////////////////////////////////////////////////////////////////
// CDXInput Create
//////////////////////////////////////////////////////////////////////////////////
BOOL CDXInput::Create(void *hInst, void *hWnd)
{
	HRESULT rval;
	MMRESULT mr;

	rval = DirectInputCreate(hInst, DIRECTINPUT_VERSION, &m_lpDI, NULL);
	if(rval != DI_OK) return FALSE;

	m_lpDI->CreateDevice(GUID_SysMouse, &m_lpDIDMouse, NULL);
	m_lpDI->CreateDevice(GUID_SysKeyboard, &m_lpDIDKeyboard, NULL);

	m_lpDIDMouse->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);
	m_lpDIDKeyboard->SetCooperativeLevel(hWnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

	m_lpDIDMouse->SetDataFormat(&c_dfDIMouse);
	m_lpDIDKeyboard->SetDataFormat(&c_dfDIKeyboard);

	rval = m_lpDIDMouse->Acquire();
	if(rval != DI_OK) return FALSE;

	rval = m_lpDIDKeyboard->Acquire();
	if(rval != DI_OK) return FALSE;

	bJoystick = joyGetNumDevs() > 0 ? TRUE : FALSE;

	memset(&JoyInfo, 0, sizeof(JoyInfo));
	JoyInfo.dwSize = sizeof(JoyInfo);
	JoyInfo.dwFlags = JOY_RETURNALL;
	mr = joyGetPosEx(0, &JoyInfo);
	if (mr != JOYERR_NOERROR)	bJoystick = FALSE;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////
// CDXInput Restore
//////////////////////////////////////////////////////////////////////////////////
void CDXInput::Restore(void)
{
	m_lpDIDMouse->Acquire();
	m_lpDIDKeyboard->Acquire();
}

//////////////////////////////////////////////////////////////////////////////////
// CDXInput Release
//////////////////////////////////////////////////////////////////////////////////
void CDXInput::Release(void)
{
	m_lpDIDMouse->Unacquire();
	m_lpDIDKeyboard->Unacquire();
}

/////////////////////////////////////////////////////////////////////////////////////////////
// CDXInput Update
/////////////////////////////////////////////////////////////////////////////////////////////
void CDXInput::Update(void)
{
	DIMOUSESTATE MouseState;
	MMRESULT mr;

	if(m_lpDIDKeyboard->GetDeviceState(256, &Keys) == (DIERR_INPUTLOST | DIERR_NOTACQUIRED))
	{
		Restore();
	}

	if(m_lpDIDMouse->GetDeviceState(sizeof(MouseState), &MouseState) == (DIERR_INPUTLOST | DIERR_NOTACQUIRED))
	{
		Restore();
	}

	JoystickB1 = FALSE;
	JoystickB2 = FALSE;

  Mouse.x = MouseState.lX;
  Mouse.y = MouseState.lY;

	MouseLB = MouseState.rgbButtons[0];
	MouseRB = MouseState.rgbButtons[1];
	MouseMB = MouseState.rgbButtons[2];

	if(bJoystick)
	{
		memset(&JoyInfo, 0, sizeof(JoyInfo));
		JoyInfo.dwSize = sizeof(JoyInfo);
		JoyInfo.dwFlags = JOY_RETURNALL;

		mr = joyGetPosEx(0, &JoyInfo);
		if (mr != JOYERR_NOERROR)	bJoystick = FALSE;

		Joystick.x = JoyInfo.dwXpos;
		Joystick.y = JoyInfo.dwYpos;
		if (JoyInfo.dwButtonNumber == 0) JoystickB1 = TRUE;
		if (JoyInfo.dwButtonNumber == 1) JoystickB2 = TRUE;
	}
}
